@@ -6,12 +6,10 @@ use std::sync::Arc;
 
 use smol_str::SmolStr;
 
-use crate::{
-    dpi::{PhysicalPosition, PhysicalSize},
-    keyboard::Key,
-};
+use crate::dpi::{PhysicalPosition, PhysicalSize};
+use crate::keyboard::Key;
 
-pub use self::event_loop::{EventLoop, EventLoopProxy, EventLoopWindowTarget};
+pub(crate) use self::event_loop::{ActiveEventLoop, EventLoop, EventLoopProxy, OwnedDisplayHandle};
 mod event_loop;
 
 pub use self::window::Window;
@@ -105,9 +103,7 @@ pub struct WindowId {
 
 impl WindowId {
     pub const fn dummy() -> Self {
-        WindowId {
-            fd: u64::max_value(),
-        }
+        WindowId { fd: u64::MAX }
     }
 }
 
@@ -133,7 +129,7 @@ impl DeviceId {
 }
 
 #[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
-pub struct PlatformSpecificWindowBuilderAttributes;
+pub struct PlatformSpecificWindowAttributes;
 
 struct WindowProperties<'a> {
     flags: &'a str,
@@ -149,31 +145,16 @@ impl<'a> WindowProperties<'a> {
         // orbital:flags/x/y/w/h/t
         let mut parts = path.splitn(6, '/');
         let flags = parts.next().unwrap_or("");
-        let x = parts
-            .next()
-            .map_or(0, |part| part.parse::<i32>().unwrap_or(0));
-        let y = parts
-            .next()
-            .map_or(0, |part| part.parse::<i32>().unwrap_or(0));
-        let w = parts
-            .next()
-            .map_or(0, |part| part.parse::<u32>().unwrap_or(0));
-        let h = parts
-            .next()
-            .map_or(0, |part| part.parse::<u32>().unwrap_or(0));
+        let x = parts.next().map_or(0, |part| part.parse::<i32>().unwrap_or(0));
+        let y = parts.next().map_or(0, |part| part.parse::<i32>().unwrap_or(0));
+        let w = parts.next().map_or(0, |part| part.parse::<u32>().unwrap_or(0));
+        let h = parts.next().map_or(0, |part| part.parse::<u32>().unwrap_or(0));
         let title = parts.next().unwrap_or("");
-        Self {
-            flags,
-            x,
-            y,
-            w,
-            h,
-            title,
-        }
+        Self { flags, x, y, w, h, title }
     }
 }
 
-impl<'a> fmt::Display for WindowProperties<'a> {
+impl fmt::Display for WindowProperties<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
@@ -198,6 +179,9 @@ impl Display for OsError {
     }
 }
 
+pub(crate) use crate::cursor::{
+    NoCustomCursor as PlatformCustomCursor, NoCustomCursor as PlatformCustomCursorSource,
+};
 pub(crate) use crate::icon::NoIcon as PlatformIcon;
 
 #[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
@@ -225,11 +209,11 @@ impl MonitorHandle {
         None
     }
 
-    pub fn video_modes(&self) -> impl Iterator<Item = VideoMode> {
+    pub fn video_modes(&self) -> impl Iterator<Item = VideoModeHandle> {
         let size = self.size().into();
         // FIXME this is not the real refresh rate
         // (it is guaranteed to support 32 bit color though)
-        std::iter::once(VideoMode {
+        std::iter::once(VideoModeHandle {
             size,
             bit_depth: 32,
             refresh_rate_millihertz: 60000,
@@ -239,14 +223,14 @@ impl MonitorHandle {
 }
 
 #[derive(Clone, Debug, Eq, Hash, PartialEq)]
-pub struct VideoMode {
+pub struct VideoModeHandle {
     size: (u32, u32),
     bit_depth: u16,
     refresh_rate_millihertz: u32,
     monitor: MonitorHandle,
 }
 
-impl VideoMode {
+impl VideoModeHandle {
     pub fn size(&self) -> PhysicalSize<u32> {
         self.size.into()
     }
